import 'package:backend_dart/application/service/dto/user_dto.dart';
import 'package:backend_dart/domain/entities/user.dart';
import 'package:backend_dart/domain/interface/error.dart';
import 'package:backend_dart/domain/interface/mapper.dart';
import 'package:fpdart/fpdart.dart';

class UserDtoMapper implements IMapper<User, UserDto> {
  @override
  TaskEither<IError, UserDto> to(User entity) => TaskEither.of(UserDto(
        id: entity.id,
        name: entity.name,
        email: entity.email,
        createdAt: entity.createdAt,
        updatedAt: entity.updatedAt,
      ));
  @override
  TaskEither<IError, User> from(UserDto dto) => TaskEither.of(User(
        id: dto.id,
        name: dto.name,
        email: dto.email,
        createdAt: dto.createdAt,
        updatedAt: dto.updatedAt,
      ));

  @override
  TaskEither<IError, List<User>> listFrom(Iterable<UserDto> targets) {
    return TaskEither.traverseList(targets.toList(), from);
  }

  @override
  TaskEither<IError, List<UserDto>> listTo(Iterable<User> origins) {
    return TaskEither.traverseList(origins.toList(), to);
  }

  TaskEither<IError, UserCreate> fromCreateTo(UserCreateDto origin) =>
      TaskEither.of(UserCreate(
        name: origin.name,
        email: origin.email,
        password: origin.password,
      ));

  TaskEither<IError, UserUpdate> fromUpdateTo(
          UserUpdateDto origin, String id) =>
      TaskEither.of(UserUpdate(
        id: id,
        name: origin.name,
        email: origin.email,
        password: origin.password,
      ));
}
